@@ -6,16 +6,16 @@ Pandas indexing helper.
 from importlib.metadata import version as _version
 
 from . import core, datasets
+from .arithmetics import add, divide, multiply, subtract
 from .core import (
     alignlevel,
     alignlevels,
     assignlevel,
     index_names,
-    isin,
-    ismatch,
     projectlevel,
     semijoin,
 )
+from .selectors import isin, ismatch
 
 
 try:
